@@ -1,21 +1,21 @@
 """
 This module provides mosaik specific exception types.
-
 """
 
 
 class ScenarioError(Exception):
-    """This exception is raised if something fails during the creationg of
+    """
+    This exception is raised if something fails during the creation of
     a scenario.
-
     """
 
 
 class SimulationError(Exception):
-    """This exception is raised if a simulator cannot be started or if
-    a problem arises during the execution of a simulation.
-
     """
+    This exception is raised if a simulator cannot be started or if
+    a problem arises during the execution of a simulation.
+    """
+
     def __init__(self, msg, exc=None):
         arg = ''
         if exc:
@@ -25,3 +25,15 @@ class SimulationError(Exception):
             arg += '%s: ' % orig
         arg += msg
         super().__init__(arg)
+
+
+class WakeUpException(Exception):
+    """
+    This exception is raised if a simulator is woken up by a successor
+    which has terminated.
+    """
+
+class NoStepException(Exception):
+    """
+    This exception is raised if none of the simulators has a next step.
+    """
