@@ -4,51 +4,50 @@ This module contains some utility functions and classes.
 """
 import random
 import sys
+from loguru import logger
 
 from simpy.io.network import RemoteException
 
 from mosaik.exceptions import SimulationError
 
 
-def sync_process(generator, world, *, ignore_errors=False):
-    """Synchronously execute a SimPy process defined by the generator object
+def sync_process(generator, world, *, errback=None, ignore_errors=False):
+    """
+    Synchronously execute a SimPy process defined by the generator object
     *generator*.
 
     A *world* instance is required to run the event loop.
 
-    You can optionally provide a *err_msg* that will be printed when the
-    remote site unexpectedly closes its connection.
+    You can optionally provide a *errback* (error callback) which will be
+    called with no arguments if an error occurs.
 
     If *ignore_errors* is set to ``True``, no errors will be printed.
-
     """
     try:
         return world.env.run(until=world.env.process(generator))
     except (ConnectionError, RemoteException, SimulationError) as exc:
+        if errback is not None:
+            errback()
+
         if ignore_errors:
             # Avoid endless recursions when called from "world.shutdown()"
             return
 
-        if type(exc) is RemoteException:
-            print('RemoteException:')
-            print(exc.remote_traceback)
-            print('————————————————')
-        else:
-            print('ERROR: %s' % exc)
+        logger.exception(exc)
 
-        print('Mosaik terminating')
+        logger.info('Mosaik terminating.')
         world.shutdown()
         sys.exit(1)
 
 
 def sync_call(sim, funcname, args, kwargs):
-    """Start a SimPy process to make the *func()* call to a simulator behave
+    """
+    Start a SimPy process to make the *func()* call to a simulator behave
     like it was synchronous.
 
     Return the result of the *func()* call.
 
     Raise an :exc:`~mosaik.exceptions.SimulationError` if an exception occurs.
-
     """
     # We have to start a SimPy process to make the "create()" call
     # behave like it was synchronous.
@@ -66,11 +65,11 @@ def sync_call(sim, funcname, args, kwargs):
 
 
 def connect_many_to_one(world, src_set, dest, *attrs, async_requests=False):
-    """:meth:`~mosaik.scenario.World.connect` each entity in *src_set*
+    """
+    :meth:`~mosaik.scenario.World.connect` each entity in *src_set*
     to *dest*.
 
     See the :meth:`~mosaik.scenario.World.connect` for more details.
-
     """
     for src in src_set:
         world.connect(src, dest, *attrs, async_requests=async_requests)
@@ -78,7 +77,8 @@ def connect_many_to_one(world, src_set, dest, *attrs, async_requests=False):
 
 def connect_randomly(world, src_set, dest_set, *attrs, evenly=True,
                      max_connects=float('inf')):
-    """Randomly :meth:`~mosaik.scenario.World.connect` the entities from
+    """
+    Randomly :meth:`~mosaik.scenario.World.connect` the entities from
     *src_set* to the entities from *dest_set* and return a subset of *dest_set*
     containing all entities with a connection.
 
@@ -108,7 +108,6 @@ def connect_randomly(world, src_set, dest_set, *attrs, evenly=True,
     *max_connects* lets you set the maximum number of connections that an
     entity of *dest_set* may receive. This argument is only taken into account
     if *evenly* is set to ``False``.
-
     """
     dest_set = list(dest_set)
     assert dest_set
